From b84af8925c44e3c40ecbbb9104f01f7adccf79be Mon Sep 17 00:00:00 2001
From: Jack Morgenstein <[EMAIL PROTECTED]>
Date: Tue, 8 Jul 2008 09:42:47 +0300
Subject: [PATCH] IB/core: implement kernel-space XRC.

V2: no changes

Signed-off-by: Jack Morgenstein <[EMAIL PROTECTED]>
---
 drivers/infiniband/core/verbs.c |   56 +++++++++++++++++++++++++++++++++++++-
 include/rdma/ib_verbs.h         |   34 +++++++++++++++++++++--
 2 files changed, 85 insertions(+), 5 deletions(-)

diff --git a/drivers/infiniband/core/verbs.c b/drivers/infiniband/core/verbs.c
index 1b60806..41dddfa 100644
--- a/drivers/infiniband/core/verbs.c
+++ b/drivers/infiniband/core/verbs.c
@@ -244,6 +244,36 @@ struct ib_srq *ib_create_srq(struct ib_pd *pd,
 }
 EXPORT_SYMBOL(ib_create_srq);
 
+struct ib_srq *ib_create_xrc_srq(struct ib_pd *pd,
+                                struct ib_cq *xrc_cq,
+                                struct ib_xrcd *xrcd,
+                                struct ib_srq_init_attr *srq_init_attr)
+{
+       struct ib_srq *srq;
+
+       if (!pd->device->create_xrc_srq)
+               return ERR_PTR(-ENOSYS);
+
+       srq = pd->device->create_xrc_srq(pd, xrc_cq, xrcd, srq_init_attr, NULL);
+
+       if (!IS_ERR(srq)) {
+               srq->device        = pd->device;
+               srq->pd            = pd;
+               srq->uobject       = NULL;
+               srq->event_handler = srq_init_attr->event_handler;
+               srq->srq_context   = srq_init_attr->srq_context;
+               srq->xrc_cq        = xrc_cq;
+               srq->xrcd          = xrcd;
+               atomic_inc(&pd->usecnt);
+               atomic_inc(&xrcd->usecnt);
+               atomic_inc(&xrc_cq->usecnt);
+               atomic_set(&srq->usecnt, 0);
+       }
+
+       return srq;
+}
+EXPORT_SYMBOL(ib_create_xrc_srq);
+
 int ib_modify_srq(struct ib_srq *srq,
                  struct ib_srq_attr *srq_attr,
                  enum ib_srq_attr_mask srq_attr_mask)
@@ -308,12 +338,15 @@ struct ib_qp *ib_create_qp(struct ib_pd *pd,
                qp->event_handler = qp_init_attr->event_handler;
                qp->qp_context    = qp_init_attr->qp_context;
                qp->qp_type       = qp_init_attr->qp_type;
-               qp->xrcd          = NULL;
+               qp->xrcd          = qp->qp_type == IB_QPT_XRC ?
+                       qp_init_attr->xrc_domain : NULL;
                atomic_inc(&pd->usecnt);
                atomic_inc(&qp_init_attr->send_cq->usecnt);
                atomic_inc(&qp_init_attr->recv_cq->usecnt);
                if (qp_init_attr->srq)
                        atomic_inc(&qp_init_attr->srq->usecnt);
+               if (qp->qp_type == IB_QPT_XRC)
+                       atomic_inc(&qp->xrcd->usecnt);
        }
 
        return qp;
@@ -644,6 +677,7 @@ int ib_destroy_qp(struct ib_qp *qp)
        struct ib_cq *scq, *rcq;
        struct ib_srq *srq;
        struct ib_xrcd *xrcd;
+       enum ib_qp_type qp_type = qp->qp_type;
        int ret;
 
        pd  = qp->pd;
@@ -659,7 +693,7 @@ int ib_destroy_qp(struct ib_qp *qp)
                atomic_dec(&rcq->usecnt);
                if (srq)
                        atomic_dec(&srq->usecnt);
-               if (xrcd)
+               if (qp_type == IB_QPT_XRC)
                        atomic_dec(&xrcd->usecnt);
        }
 
@@ -978,4 +1012,22 @@ int ib_dealloc_xrcd(struct ib_xrcd *xrcd)
 }
 EXPORT_SYMBOL(ib_dealloc_xrcd);
 
+struct ib_xrcd *ib_alloc_xrcd(struct ib_device *device)
+{
+       struct ib_xrcd *xrcd;
+
+       if (!device->alloc_xrcd)
+               return ERR_PTR(-ENOSYS);
+
+       xrcd = device->alloc_xrcd(device, NULL, NULL);
+       if (!IS_ERR(xrcd)) {
+               xrcd->device = device;
+               xrcd->inode = NULL;
+               xrcd->uobject = NULL;
+               atomic_set(&xrcd->usecnt, 0);
+       }
+       return xrcd;
+}
+EXPORT_SYMBOL(ib_alloc_xrcd);
+
 
diff --git a/include/rdma/ib_verbs.h b/include/rdma/ib_verbs.h
index 774b4e2..cf8b594 100644
--- a/include/rdma/ib_verbs.h
+++ b/include/rdma/ib_verbs.h
@@ -758,6 +758,7 @@ struct ib_send_wr {
                        u32                             rkey;
                } fast_reg;
        } wr;
+       u32                     xrc_remote_srq_num; /* valid for XRC sends only 
*/
 };
 
 struct ib_recv_wr {
@@ -883,6 +884,7 @@ struct ib_srq {
        void                  (*event_handler)(struct ib_event *, void *);
        void                   *srq_context;
        atomic_t                usecnt;
+       u32                     xrc_srq_num;
 };
 
 struct ib_qp {
@@ -1340,8 +1342,28 @@ int ib_query_ah(struct ib_ah *ah, struct ib_ah_attr 
*ah_attr);
 int ib_destroy_ah(struct ib_ah *ah);
 
 /**
- * ib_create_srq - Creates a SRQ associated with the specified protection
- *   domain.
+ * ib_create_xrc_srq - Creates an XRC SRQ associated with the specified
+ *   protection domain, cq, and xrc domain.
+ * @pd: The protection domain associated with the SRQ.
+ * @xrc_cq: The cq to be associated with the XRC SRQ.
+ * @xrcd: The XRC domain to be associated with the XRC SRQ.
+ * @srq_init_attr: A list of initial attributes required to create the
+ *   XRC SRQ.  If XRC SRQ creation succeeds, then the attributes are updated
+ *   to the actual capabilities of the created XRC SRQ.
+ *
+ * srq_attr->max_wr and srq_attr->max_sge are read the determine the
+ * requested size of the XRC SRQ, and set to the actual values allocated
+ * on return.  If ib_create_xrc_srq() succeeds, then max_wr and max_sge
+ * will always be at least as large as the requested values.
+ */
+struct ib_srq *ib_create_xrc_srq(struct ib_pd *pd,
+                                struct ib_cq *xrc_cq,
+                                struct ib_xrcd *xrcd,
+                                struct ib_srq_init_attr *srq_init_attr);
+
+/**
+ * ib_create_srq - Creates an SRQ associated with the specified
+ *   protection domain.
  * @pd: The protection domain associated with the SRQ.
  * @srq_init_attr: A list of initial attributes required to create the
  *   SRQ.  If SRQ creation succeeds, then the attributes are updated to
@@ -2062,8 +2084,14 @@ int ib_detach_mcast(struct ib_qp *qp, union ib_gid *gid, 
u16 lid);
 
 /**
  * ib_dealloc_xrcd - Deallocates an extended reliably connected domain.
  * @pd: The xrc domain to deallocate.
  */
 int ib_dealloc_xrcd(struct ib_xrcd *xrcd);
 
+/**
+ * ib_alloc_xrcd - Allocates an extended reliably connected domain.
+ * @device: The device on which to allocate the xrcd.
+ */
+struct ib_xrcd *ib_alloc_xrcd(struct ib_device *device);
+
 #endif /* IB_VERBS_H */
@@ -2062,8 +2084,14 @@ int ib_detach_mcast(struct ib_qp *qp, union ib_gid *gid, 
 /**
  * ib_dealloc_xrcd - Deallocates an extended reliably connected domain.
- * @pd: The xrc domain to deallocate.
+ * @xrcd: The xrc domain to deallocate.
  */
 int ib_dealloc_xrcd(struct ib_xrcd *xrcd);
-- 
1.5.1.6

_______________________________________________
general mailing list
general@lists.openfabrics.org
http://lists.openfabrics.org/cgi-bin/mailman/listinfo/general

To unsubscribe, please visit http://openib.org/mailman/listinfo/openib-general

Reply via email to